#include <BoolField.h>
#include <DegreeField.h>
#include <Field.h>
#include <Form.h>
#include <IntField.h>
#include <LCD.h>
#include <ListField.h>
#include <TextField.h>
#include <TimeField.h>
#include <GetIntField.h>
#include <TimerOne.h>
#include <EEPROM.h>
#include "EEPROMAnything.h"

void goDivide(long gAngle);
void goJog(void);
void enterDegrees(void);
void enterDivisions(void);
void goSetSpeed(void);
void goHome(void);
void goSetJogStep(void);
void setSpeed(int spdPct);
void goConTurn(void);
long HzToTimeDelay(long iHz);
long TimeDelayToHz(long tDelay);
int getGlobals(int gDevice);
int putGlobals(void);
int getCongiguration(void);
int putConfiguration(void);
void doSetupMenu(void);
void setSteps(long sSteps, int sWait);
void rampDown(void);

#define STEP_PIN             1
#define DIR_PIN              2
#define CLOCKWISE            LOW
#define ANTICLOCKWISE        HIGH
#define MINUTES_IN_CIRCLE    1296000      // Number od seconds in a 360 degree circle
#define MAX_LONG             2147483646   // The biggest number we can use with a long
#define CONFIG_VERSION       "Rot"        // The Sigature to tell us we've saved stuff in the EEPROM
#define STEPPER_STEPS_REV    200          // The number of steps per stepper
#define MICRO_STEPS          10           // set to 1 if not not using a micostepper driver, otherwise set to # microsteps
#define DIVIDE_RATIO         90           // The Dividing table ratio (eg 40 or 90)
#define MAX_STEPPER_HZ       1000         // Maximum frequency for this stepper
#define MIN_STEPPER_HZ       400          // Minimum frequency for this stepper
#define TIMER_DELAY          80           // 80 MIcroseconds between interrupt ticks
#define JOG_STEP_ANGLE       1800L        // 0.5 degrees (30 seconds)

struct globals_t
{
  int  DirPin;
  int  StepPin;
  long stepperStepsPerRev;
  int  microSteps;
  int  divideRatio;
  long maxStepperHz;
  long minStepperHz;
  long stepsPerRev;
  long JogStepAngle;
  long JogStepCount;
  int  dir;
  long timerDelay;
} globals={
    DIR_PIN,
    STEP_PIN,
    (long)STEPPER_STEPS_REV,
    MICRO_STEPS,
    DIVIDE_RATIO,
    (long)MAX_STEPPER_HZ,
    (long)MIN_STEPPER_HZ,
    ((long)STEPPER_STEPS_REV * (long)MICRO_STEPS * (long)DIVIDE_RATIO),    
    (long)JOG_STEP_ANGLE,
    (long)(((float)STEPPER_STEPS_REV * (float)MICRO_STEPS * (float)DIVIDE_RATIO) *((float)JOG_STEP_ANGLE/(float)MINUTES_IN_CIRCLE)),
    CLOCKWISE, 
    TIMER_DELAY
};

struct config_t
{
  char version[4]; 
  int currDevice;
}configuration;



//end of what should be stored in gobal variables.

LCD lcd;
volatile int state = CLOCKWISE;
volatile long steps = 0L;      // Number of steps to process
volatile long stepcount = 0;  // Number of steps done;
long rampSteps;
volatile int stepmod=0;
volatile int mSteps=0; // microsteps

volatile long endRampUp;       // Steps to stop ramping up
volatile long startRampDown;   // Steps to start ramping down
static long startDelay;        // Start delay - slow speed stepper start pulse delay in microseconds 
static long targetDelay;       // Target delay - Full speed stepper pulse delay in microseconds
static long sDelay;            // Current delay for stepper movement

volatile long oldSpeed;        // Remember the old speed before we play with it
volatile long homePos = 0L;    // Remember the home position
long divisions=3;
long stepsPerDiv=0;
long mAngle = 0L;
int goinghome = 0;  // True if Table is going home
long MaxOperating_Hz = (long)globals.maxStepperHz * (long)globals.microSteps;
long MinOperating_Hz = (long)globals.maxStepperHz * (long)globals.microSteps;
long Operating_Hz;  // The frequency the steper is operating at

const char item_Start[] PROGMEM = "UP or DWN Starts";
 ListItem const DivideMenu[] PROGMEM = {
    item_Start,
    0
 };
 ListItem const JogMenu[] PROGMEM = {
    item_Start,
    0
 };
 ListItem const AngleMenu[] PROGMEM = {
    item_Start,
    0
 };
 ListItem const HomeMenu[] PROGMEM = {
    item_Start,
    0
 };
 ListItem const SpeedMenu[] PROGMEM = {
    item_Start,
    0
 };
 ListItem const JogStepMenu[] PROGMEM = {
    item_Start,
    0
 };

 ListItem const TurnMenu[] PROGMEM = {
    item_Start,
    0
 };

 ListItem const MainSetupMenu[] PROGMEM = {
    item_Start,
    0
 };

Form mainForm(lcd);
ListField goDivideField(mainForm, "   <Divide #>" , DivideMenu);
ListField goAngleField(mainForm, " <Divide Angle>" , AngleMenu);
ListField goJogField(mainForm,"     <Jog>" , JogMenu);
ListField goJogStepField(mainForm,"   <Jog Step>" ,JogStepMenu);
ListField goHomeField(mainForm, "     <Home>" ,HomeMenu);
//ListField goSpeedField(mainForm,"  <Set Speed>" ,SpeedMenu);
ListField goConTurnField(mainForm," <Contin Turn>",TurnMenu);
ListField goSetupField(mainForm,"    <Setup>" ,MainSetupMenu);

void setup() 
{
   const char welcome1[] PROGMEM = "VMN Rotary Table";
   const char welcome2[] PROGMEM = "Controller 2015";
   long previousMillis = 0;
   unsigned long currentMillis;
   long interval = 2000L;
   int event = LCD_BUTTON_NONE;

  //read the config from EEPROM
//   Serial.begin(9600);
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz);  
  sDelay = targetDelay;

//  rampDown();

   EEPROM_readAnything(0, configuration);
   {
     const char sig[] PROGMEM = CONFIG_VERSION;
     int ok = 1;
     int cs = sizeof(configuration);
     char *s = (char *)sig;
     char *p= (char *)&configuration;
     while(*s){
       if(*s != *p){
         ok = 0;
         break;
       }
       s++; p++; 
     }
     if(!ok){     
       // No data in EEprom, so set the data
       strcpy(configuration.version,sig);
       configuration.currDevice = 0;
       EEPROM_writeAnything(0, configuration); // write to EEProm
       EEPROM_writeAnything(cs,globals); // Write the defaults to EEprom starting at the first byte after configuration
     }
     else{
       getGlobals(configuration.currDevice);
     }
   }   mAngle =(long)((float)MINUTES_IN_CIRCLE / (float)divisions);
   float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV
   stepsPerDiv = (long)(tmpSteps *((float)mAngle/(float)MINUTES_IN_CIRCLE));
   globals.JogStepCount  = (long)(tmpSteps *((float)globals.JogStepAngle/(float)MINUTES_IN_CIRCLE)) * 2L;
   pinMode(globals.StepPin, OUTPUT);
   pinMode(globals.DirPin, OUTPUT);

   Timer1.initialize(globals.timerDelay); // set a timer of length in microseconds (or 0.1 sec - or 10Hz)
   Timer1.attachInterrupt( timerIsr ); // attach the service routine here
   // Enable the screen saver, which will automatically blank the screen after 30 seconds.
   // The screen will wake up again when a button is pressed or lcd.display() is called.
   // You may wish to comment this line out if it annoys you
   lcd.enableScreenSaver(30);
   Operating_Hz =  TimeDelayToHz(globals.timerDelay); 
   // Print a welcome message  

   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(welcome1);
   lcd.setCursor(0,1);
   lcd.print(welcome2); 

   while(1){  // Delay 2 seconds without using delay();
     currentMillis = millis();
     if(currentMillis - previousMillis > interval) {
        previousMillis = currentMillis;
        break;  
     }
     event = lcd.getButton();
/*
     if(event == LCD_BUTTON_SELECT){
       doSetupMenu();
       break;
     }
*/
   }
   lcd.clear();
 
  // Show the main form for the first time.
  mainForm.show();
}
 
void loop()
{
  int i;
  // Dispatch button events to the main form.
  int event = lcd.getButton();
  if (mainForm.dispatch(event) == FORM_CHANGED) {
    if (mainForm.isCurrent(goDivideField)) {
      enterDivisions();
      mainForm.setCurrentField(&goDivideField);
//      mainForm.show();
    }
    if (mainForm.isCurrent(goJogField)) {
      goJog();
      mainForm.setCurrentField(&goJogField);
    }
    if (mainForm.isCurrent(goAngleField)) {
      enterDegrees();
      mainForm.setCurrentField(&goAngleField);
    }
    if (mainForm.isCurrent(goHomeField)) {
      goHome();
      mainForm.setCurrentField(&goHomeField);
    }
/*
    if (mainForm.isCurrent(goSpeedField)) {
      goSetSpeed();
      mainForm.setCurrentField(&goSpeedField);
    }    
*/
    if (mainForm.isCurrent(goJogStepField)) {
      goSetJogStep();
      mainForm.setCurrentField(&goJogStepField);
    }
    if (mainForm.isCurrent(goConTurnField)) {
      goConTurn();
      mainForm.setCurrentField(&goConTurnField);
    }
    if (mainForm.isCurrent(goSetupField)) {
      doSetupMenu();
      mainForm.setCurrentField(&goSetupField);
    }
  }
}
 
void printDivideVals(int iDiv,long iAngle)
{
  Form tempForm(lcd);
  DegreeField tempField(tempForm, "Tmp", 360, DEGREEFIELD_READ_WRITE);

  lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Div: ");
   lcd.print(iDiv);
   lcd.print(" of ");
   lcd.print(divisions);
   tempField.setValue(iAngle);
   tempField.printDegrees();
   lcd.setCursor(10,1);
   lcd.print(" / Div");  
}


void goDivide(long gAngle)
{
   long thisDiv = 1; 
   int event = LCD_BUTTON_NONE;
   float tmpSTEPS_PER_REV = (float)globals.stepsPerRev;
   
   stepsPerDiv = (long)(tmpSTEPS_PER_REV *((float)gAngle/(float)MINUTES_IN_CIRCLE));
   divisions = (long)((float) MINUTES_IN_CIRCLE / (float) gAngle);
   printDivideVals(thisDiv,gAngle);
   while(event != LCD_BUTTON_SELECT){
      event = lcd.getButton();
      switch(event){
        case LCD_BUTTON_LEFT:
          thisDiv--;
          if(thisDiv < 1)
            thisDiv = divisions;          
          while(steps){
            ;
          }
          digitalWrite( globals.DirPin, ANTICLOCKWISE );
          setSteps(stepsPerDiv * 2, 1);
          printDivideVals(thisDiv,gAngle);
          break;
        case LCD_BUTTON_RIGHT:
          thisDiv++;
          if(thisDiv > divisions)
            thisDiv = 1;          
          while(steps){
            ;
          }
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(stepsPerDiv * 2,1); 
          printDivideVals(thisDiv, gAngle);
          break;
    } 
  }
}

void goJog(void)
{
   int event,event2 = LCD_BUTTON_NONE;
   const char jog1[] PROGMEM = "CONT: Left/Right";
   const char jog2[] PROGMEM = "STEP: Up/Down";

   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(jog1);
   lcd.setCursor(0,1);
   lcd.print(jog2); 
   
   while(event != LCD_BUTTON_SELECT){
      event = lcd.getButton();
      if(sDelay < HzToTimeDelay(globals.maxStepperHz))
        sDelay = HzToTimeDelay(globals.maxStepperHz);
      switch(event){
        case LCD_BUTTON_LEFT:
          digitalWrite( globals.DirPin, ANTICLOCKWISE );
          setSteps(MAX_LONG, 0);
          break;
        case LCD_BUTTON_RIGHT:
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(MAX_LONG,0);  
          break;
        case LCD_BUTTON_UP:
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(globals.JogStepCount, 1);
          break;
        case LCD_BUTTON_DOWN:
          digitalWrite( globals.DirPin, ANTICLOCKWISE );
          setSteps(globals.JogStepCount, 1);
          break;
        case LCD_BUTTON_LEFT_RELEASED:
        case LCD_BUTTON_RIGHT_RELEASED:
          steps = 0;       
         // quitWhile ++;
          rampDown();    // Need to add enough steps to slow down
          break;
       }
    } 
    steps = 0L;
}

void enterDegrees(void)
{
  
  Form angleForm(lcd);
  DegreeField angleField(angleForm,"Set Angle" , 360, DEGREEFIELD_READ_WRITE);
  
  int event = LCD_BUTTON_NONE;

  angleField.setValue(mAngle);
  angleForm.defaultField();
  angleForm.show();
  while(event!= LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if(event == LCD_BUTTON_SELECT)
      break;
    if (angleForm.dispatch(event) == FORM_CHANGED) {
        if (angleForm.isCurrent(angleField)) {
          mAngle = angleField.value();
          float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV
          divisions = (long)(tmpSteps *((float)mAngle/(float)MINUTES_IN_CIRCLE));
        }
    }
  }
  if(mAngle)
    goDivide(mAngle);
}

void enterDivisions(void)
{
  Form divideForm(lcd);
  GetIntField divfField(divideForm,"Divisions" , 9999, GETINTFIELD_READ_WRITE);
  
  int event = LCD_BUTTON_NONE;
  divfField.setValue(divisions);
  divideForm.defaultField();
  divideForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (divideForm.dispatch(event) == FORM_CHANGED) {
        if (divideForm.isCurrent(divfField)) {
          divisions = divfField.value();        
          mAngle=(long)((float)MINUTES_IN_CIRCLE / (float)divisions);
       }
    }
  }
  if(divisions)
    goDivide(mAngle);
}

void goSetSpeed(void)
{
  Form speedForm(lcd);
  IntField speedField(speedForm,"Speed" , 0, 100, 5, 85, "%");

  //  RJW - why is the speed not initialising the percent value?
  long currentHz = TimeDelayToHz(globals.timerDelay);
  long delayrange = (MaxOperating_Hz - MinOperating_Hz);
  int speedPct = (int)((((float)(currentHz - MinOperating_Hz) / (float)delayrange))*100.0);
  int event = LCD_BUTTON_NONE;
  lcd.clear();
  speedField.setValue(speedPct);
  speedForm.defaultField();
  speedForm.show();
  while(event != LCD_BUTTON_SELECT ){
    event = lcd.getButton();
    if (speedForm.dispatch(event) == FORM_CHANGED) {
        if (speedForm.isCurrent(speedField)) {
          speedPct = speedField.value();
        }
    }
  }
  setSpeed(speedPct);
}

void setSpeed(int spdPct)
{
  long mindelay =  HzToTimeDelay(MaxOperating_Hz);
  long maxdelay =  HzToTimeDelay(MinOperating_Hz);

  long delayrange = maxdelay - mindelay;
  float changeRatio = (float)spdPct/100.0;
  long offset = (long)((float)delayrange * changeRatio);
  long newspeed = maxdelay - offset;  // RJW Check this here
  globals.timerDelay = newspeed; // save globally
  Timer1.setPeriod(globals.timerDelay); // set a timer of length in microseconds
  //  RJW - we should save the globals here
  //  RJW - why 
}

void goHome(void)
{
   int event = LCD_BUTTON_NONE;
   long tmpSteps = (long)globals.stepsPerRev;
   
   const char home1[] PROGMEM = "SET:  Left/Right";
   const char home2[] PROGMEM = "GOTO: Up/Down";
   const char home3[] PROGMEM = "HOME Set  ";
   const char home4[] PROGMEM =       "Going HOME";
 
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print(home1);
   lcd.setCursor(0,1);
   lcd.print(home2); 

   while(1){
      event = lcd.getButton();
      if(event == LCD_BUTTON_SELECT)
        break;
      switch(event){
        //set home
        case LCD_BUTTON_LEFT:
        case LCD_BUTTON_RIGHT:
          lcd.setCursor(6,0);
          lcd.print(home3);
          while(steps){
            ;
          }
          homePos = 0;
          break;
        case LCD_BUTTON_UP:
        case LCD_BUTTON_DOWN:
        // Goto home
          lcd.setCursor(6,1);
          lcd.print(home4);
          goinghome++;
          while(steps){
            ;
          }
          if(homePos <= (tmpSteps / 2L)){
            digitalWrite( globals.DirPin, ANTICLOCKWISE );
            setSteps(homePos,1);
          }
          else{    
            digitalWrite( globals.DirPin, CLOCKWISE );
            setSteps(tmpSteps - homePos, 1);
          }
          homePos = 0;
          goinghome = 0;
          break;
       default:
         break;
      }
   }
}

void goSetJogStep(void)
{
  Form jogStepForm(lcd);
  DegreeField jogStepField(jogStepForm,"Jog Step Angle" , 10, DEGREEFIELD_READ_WRITE);

  int event = LCD_BUTTON_NONE;
  float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV

  lcd.clear();
  jogStepField.setValue(globals.JogStepAngle);
  jogStepForm.defaultField();
  jogStepForm.show();
  while(event != LCD_BUTTON_SELECT ){
    event = lcd.getButton();
    if (jogStepForm.dispatch(event) == FORM_CHANGED) {
        if (jogStepForm.isCurrent(jogStepField)) {
          globals.JogStepAngle = jogStepField.value() * 2;
          globals.JogStepCount = (long)(tmpSteps *((float)globals.JogStepAngle/(float)MINUTES_IN_CIRCLE));  
        }
    }
  }
}
  
void goConTurn(void)
{

  Form conTurnForm(lcd);
  TextField conTurnField(conTurnForm,"Continuous Turn" ,"SELECT to Stop" );

  int event = LCD_BUTTON_NONE;
  lcd.clear();
  conTurnForm.defaultField();
  conTurnForm.show();
  setSteps(MAX_LONG,0);
  while(event != LCD_BUTTON_SELECT ){
    if(sDelay < HzToTimeDelay(globals.maxStepperHz))
      sDelay = HzToTimeDelay(globals.maxStepperHz);
    if(steps < (rampSteps * 4L))     
      steps = MAX_LONG - (rampSteps * 4L);
    event = lcd.getButton();
    switch(event){
      case LCD_BUTTON_LEFT:
        if(digitalRead(globals.DirPin) != ANTICLOCKWISE){
          rampDown();
          while(steps){
            ;
          }
          digitalWrite(globals.DirPin, ANTICLOCKWISE );
          setSteps(MAX_LONG,0);
        }     
        break;
      case LCD_BUTTON_RIGHT:
        if(digitalRead( globals.DirPin) != CLOCKWISE){
          rampDown();
          while(steps){
            ;
          }
          digitalWrite( globals.DirPin, CLOCKWISE );
          setSteps(MAX_LONG, 0);
        }       
        break;
    }      
      if (conTurnForm.dispatch(event) == FORM_CHANGED) {
        if (conTurnForm.isCurrent(conTurnField)) {
        }
    }
  }
  rampDown();
  while(steps){
    ;
  }
  //steps = 0L;
}

int getGlobals(int gDevice)
{
  // Get the configuraton data
  EEPROM_readAnything(gDevice * sizeof(globals) + sizeof(configuration),globals);  // configuraaton exists, so read the curent setup.
  // DO any further setup for this device
  return 0;
}

int putGlobals(void)
{
  int offset = configuration.currDevice * sizeof(globals) + sizeof(configuration);  // calculate the offset based on setup number
  EEPROM_writeAnything(offset,globals); // Write the defaults to EEprom starting at the first byte after configuration
 
  return 0;
}

void doSetupMenu(void)
{
  Form setupForm(lcd);
//  TextField swelcomeSetupField(setupForm, "Rotary Table", "SETUP" );
  int saveSetup = false;

  IntField sDirPinField (setupForm, "Direction Port",0,20,1,2," Port");
  IntField sStepPinField (setupForm,"Stepper Port",0,20,1,1," Port");
  IntField sMicroStepsField (setupForm,"Micro Steps",0,99,1,10,":1");
  IntField sDivideRatioField (setupForm,"Dividing Ratio",0,999,1,globals.divideRatio,":1");
  GetIntField sMaxStepperHzField(setupForm, "Max Stepper Hz", 999999, GETINTFIELD_READ_WRITE);
  GetIntField sMinStepperHzField(setupForm, "Min Stepper Hz", 999999, GETINTFIELD_READ_WRITE);
  GetIntField sStepperStepsRevField(setupForm,"Stepper Steps/Rv" , 999, GETINTFIELD_READ_WRITE);
  DegreeField sJogStepAngleField(setupForm, "Jog Step Angle",360, DEGREEFIELD_READ_WRITE);
  GetIntField sTimerDelayField(setupForm, "Timer Delay (ms)", 9999, GETINTFIELD_READ_WRITE); 
  BoolField sDirField(setupForm, "Rotation", "AntiClockwise","Clockwise", false);
  BoolField sSaveField(setupForm, "Save setup?", "Yes", "No", false);

  int event = LCD_BUTTON_NONE;
  sDirPinField.setValue(globals.DirPin);
  sStepPinField.setValue(globals.StepPin);
  sStepperStepsRevField.setValue(globals.stepperStepsPerRev);
  sMicroStepsField.setValue(globals.microSteps);
  sDivideRatioField.setValue(globals.divideRatio);
  sMaxStepperHzField.setValue(globals.maxStepperHz);
  sMinStepperHzField.setValue(globals.minStepperHz);
  sJogStepAngleField.setValue(globals.JogStepAngle);
  sDirField.setValue(globals.dir);
  sTimerDelayField.setValue(globals.timerDelay);
  lcd.clear(); 
  setupForm.defaultField();
  setupForm.show();
  while(event != LCD_BUTTON_SELECT){
    event = lcd.getButton();
    if (setupForm.dispatch(event) == FORM_CHANGED) {
        if (setupForm.isCurrent(sDirPinField)) {
          globals.StepPin = sStepPinField.value();
        }
       if (setupForm.isCurrent(sDirPinField)) {
          globals.DirPin = sDirPinField.value();
        }
        if (setupForm.isCurrent(sMicroStepsField)) {
          globals.microSteps = sMicroStepsField.value();
          globals.stepsPerRev = globals.stepperStepsPerRev * globals.microSteps * globals.divideRatio;
        }
        if (setupForm.isCurrent(sStepperStepsRevField)) {
          globals.stepperStepsPerRev = sStepperStepsRevField.value();
          globals.stepsPerRev = globals.stepperStepsPerRev * globals.microSteps * globals.divideRatio;
        }
        if (setupForm.isCurrent(sDivideRatioField)) {
          globals.divideRatio = sDivideRatioField.value();
          globals.stepsPerRev = globals.stepperStepsPerRev * globals.microSteps * globals.divideRatio;
        }
        if (setupForm.isCurrent(sMaxStepperHzField)) {
          globals.maxStepperHz = sMaxStepperHzField.value();
          MaxOperating_Hz = (long)globals.maxStepperHz * (long)globals.microSteps;
        }
        if (setupForm.isCurrent(sMinStepperHzField)) {
          globals.minStepperHz = sMinStepperHzField.value();
          MinOperating_Hz = (long)globals.maxStepperHz * (long)globals.microSteps;
        }
        if (setupForm.isCurrent(sJogStepAngleField)) {
          globals.JogStepAngle = sJogStepAngleField.value();
          float tmpSteps = (float) globals.stepsPerRev;  // (float)STEPS_PER_REV
          globals.JogStepCount  = (long)(tmpSteps *((float)globals.JogStepAngle/(float)MINUTES_IN_CIRCLE)) * 2L;       
        }
        if (setupForm.isCurrent(sTimerDelayField)) {
          globals.timerDelay = sTimerDelayField.value();
          Operating_Hz =  TimeDelayToHz(globals.timerDelay); 
        }
        if (setupForm.isCurrent(sDirField)) {
          globals.dir = sDirField.value();
        }
        if (setupForm.isCurrent(sSaveField)) {
           saveSetup = sSaveField.value();         
        }
    } 
  }  
  if(saveSetup){
    int retval = putGlobals();
  }
}


long HzToTimeDelay(long iHz)
{
  long retval = 0L;
  retval = (long)((1000000.0 /(float)iHz) / 2.0);
  return(retval);
}

long TimeDelayToHz(long tDelay)
{
  long retval = 0L;
  retval = (long)(((1000000.0 /(float)tDelay)) / 2.0);
  return(retval);
}

/*
void rampUp(long sSteps)
{
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz);  
  rampSteps = ((startDelay-targetDelay) * globals.microSteps);  
  endRampUp = (sSteps - rampSteps);    // Steps to stop ramping up -- steps count down so we start at a high #
  steps = sSteps;                      // GO for it
}
*/

void rampDown(void)
{
  long temDelay;
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz); 
  temDelay = max(targetDelay,sDelay);
  rampSteps = ((startDelay-targetDelay) * globals.microSteps);  
//  rampSteps = ((startDelay-temDelay) * globals.microSteps);  
  startRampDown = rampSteps - 3L;      // Steps to start ramping down - steps count down so we start at a high #
  endRampUp =  rampSteps -1L;                      // We don't want to Ramp Up as we are supposed to be moving
//  sDelay = max(targetDelay,sDelay);                // set starting delay
  sDelay = targetDelay;
  steps = rampSteps + 5L;              // Run enough steps to stop gracefully
}

void setSteps(long sSteps, int sWait)
{
  rampSteps = (long)(((float)sSteps / 2.0));      // the number of on/off cycles 
  startDelay =   HzToTimeDelay(globals.minStepperHz);          
  targetDelay =  HzToTimeDelay(globals.maxStepperHz);  
  if(((startDelay-targetDelay) * globals.microSteps) < rampSteps) // Don't go faster than the max stepper speed
    rampSteps = ((startDelay-targetDelay) * globals.microSteps);  
  if(sSteps == MAX_LONG)
    rampSteps = (startDelay-targetDelay) * globals.microSteps;
  startRampDown = rampSteps;           // Steps to start ramping down - steps count down so we start at a high #
  endRampUp = (sSteps - rampSteps);    // Steps to stop ramping up -- steps count down so we start at a high #
  oldSpeed = sDelay;                   // save the old speed
  sDelay = startDelay;                 // set starting delay
  Timer1.initialize(sDelay);           // set the interrupt timer
  stepcount = 0L;                      // Zero the step counter
  steps = sSteps;                      // GO for it
  if(sWait){
    while(steps){
      ;                                  // Wait until done
    }
  }
  sDelay = oldSpeed; 
}

/// --------------------------
/// Custom ISR Timer Routine
/// --------------------------
void timerIsr()
{
   // Toggle Stepper
   if(steps){
      state = state ^ 1;
      digitalWrite( globals.StepPin, state );
      if(!state){
           if(steps >= endRampUp){
             stepcount++; stepcount++;
             stepmod = (long)stepcount % (long)globals.microSteps;
             if(stepmod == 0){
               sDelay--;
             Timer1.initialize(sDelay);     // set the interrupt timer 1 step slower (eg. increase the timer delay)
             }
          }
          else if(steps <= startRampDown){
             stepcount--; stepcount--;
            stepmod = ((long)stepcount % (long)globals.microSteps);
            if(stepmod == 0){
              sDelay++;
              Timer1.initialize(sDelay);     // set the interrupt timer 1 step slower (eg. increase the timer delay)
          }
        } 
      }      
      steps--;
      if(!goinghome){
        if(globals.dir == CLOCKWISE){
          homePos++;
          if(homePos > (long)globals.stepsPerRev)
            homePos = 0L;      
        } 
        else{
          homePos--;
          if(homePos < 0L)   
            homePos =  (long)globals.stepsPerRev;
        }
     }    
   }
}


